client
	New()
		. = ..()
		GetScreenboxes()
	
	proc
		GetScreenboxes()
			var global/fonts[] = list("Arial", "Trebuchet MS", "Lucida Sans Unicode", "Consolas", "Courier")
			
			// This number determines the number of boxes placed randomly in the world.
			// More screenboxes = more CPU usage. 
			// You probably shouldn't have too many screenboxes active at the same time. 
			var screenboxes[] = new (50)
			
			for(var/i in 1 to screenboxes.len)
				var screenbox/textbox = new (src,
					CenterX = rand(32 * world.maxx),
					CenterY = rand(32 * world.maxy),
					Width   = rand(32, 256),
					Height  = rand(32, 256))
				screenboxes[i] = textbox
	
				// if a textbox is more than 256 pixels offscreen, it disappears
				textbox.SetLimit(256)
	
				// textboxes stay 8 pixels away from the screen edge
				textbox.SetMargin(8, 8, 8, 8)
	
				// textboxes have a dark gray (transparent black) shadow (background)
				textbox.SetShadow(rgb(rand(64), rand(64), rand(64), rand(128, 192)), 0, 0)
	
				// size of maptext with inner text margin of 5 pixels
				textbox.maptext_width = textbox.aabb.width - 10
				textbox.maptext_height = textbox.aabb.height - 10
				textbox.maptext_x = 5
				textbox.maptext_y = 5
				textbox.layer = EFFECTS_LAYER
	
				// style of maptext (this could be its own separate variable)
				textbox.maptext = "\
					<text align=left valign=top>\
					<font size=1 color=white face='[fonts[1 + (i-1) % fonts.len]]'>"
	
				// PERFORMANCE TEST:
				// When I increase the upper bound, the framerate dies.
				for(var/n in 1 to 1) textbox.maptext += "\
					Lorem ipsum dolor sit amet, consectetur adipiscing elit. \
					Quisque sed gravida eros, sit amet posuere lectus. \
					Mauris bibendum sed leo sit amet pulvinar. \
					Nunc placerat eget lacus tempus sodales. "
	
			// screenboxes are visible off the edges of the map 
			// and over places you can't see due to opacity
			// since they're really HUD objects. 
			var screenbox/info_text = new (src, -64, 16, 128, 32)
			screenboxes += info_text
			info_text.SetLimit(0)
			info_text.maptext_width = info_text.aabb.width - 10
			info_text.maptext_height = info_text.aabb.height - 10
			info_text.maptext_x = 5
			info_text.maptext_y = 5
			info_text.maptext = "<text align=left valign=top><font size=2 color=white>"
			info_text.maptext += "<b>WASD</b> to move."



// map-related stuff

#include <kaiochao\shapes\shapes.dme>
#include "..\..\lib\keyboard.dm"

world
	fps = 60
	maxx = 100
	maxy = 100
	view = 10
	area = /area/default
	turf = /turf/default
	mob = /mob/player

client
	Move()

area/default
	invisibility = 101

turf/default
	icon_state = "rect"
	name = ""
	New() color = (x % 2 == y % 2) ? "silver" : "gray"

mob/player
	icon_state = "oval"
	color = "navy"
	step_size = 8

	Login()
		..()
		Loop()

	proc
		Loop()
			set waitfor = FALSE
			var start = world.timeofday
			while(client)
				var ix = client.GetAxis("d", "a")
				var iy = client.GetAxis("w", "s")
				ix && step(src, ix > 0 ? EAST : WEST)
				iy && step(src, iy > 0 ? NORTH : SOUTH)

				var time = round(world.time)
				var realtime = world.timeofday - start
				var dt = round(realtime - time)

				winset(src, ":window", "title=\"[world.name] ([world.cpu]%, time: [time], realtime: [realtime], dt: [dt])\"")

				sleep world.tick_lag

#if DM_VERSION < 509 && defined(DEBUG)
mob
	Login()
		..()
		client.UpdateBounds()

	Move()
		. = ..()
		client.UpdateBounds()

client
	bound_width = (10*2+1)*TILE_WIDTH
	bound_height = (10*2+1)*TILE_HEIGHT

	var
		last_view

	proc
		// rough bounds calculation
		// doesn't take EDGE_PERSPECTIVE into account, among other things
		UpdateBounds()
			if(view != last_view)
				last_view = view
				if(istext(view))
					bound_width = text2num(view)
					bound_height = text2num(copytext(view, length("[bound_width]")+2))
				else
					bound_width = (view*2+1)*TILE_WIDTH
					bound_height = (view*2+1)*TILE_HEIGHT
			var atom/e = eye
			bound_x = (e.x-1)*TILE_WIDTH  - bound_width/2  + (istype(e, /atom/movable) && e:bound_x + e:step_x)
			bound_y = (e.y-1)*TILE_HEIGHT - bound_height/2 + (istype(e, /atom/movable) && e:bound_y + e:step_y)
			bounds = list(bound_x, bound_y, bound_width, bound_height, e.z)
#endif
